@@ -1,6 +1,6 @@
 /* This implements the relp mapping onto TCP.
  *
- * Copyright 2008-2014 by Rainer Gerhards and Adiscon GmbH.
+ * Copyright 2008-2016 by Rainer Gerhards and Adiscon GmbH.
  *
  * This file is part of librelp.
  *
@@ -46,6 +46,7 @@
 #include <unistd.h>
 #include <string.h>
 #include <errno.h>
+#include <poll.h>
 #include <assert.h>
 #include "relp.h"
 #include "relpsrv.h"
@@ -420,6 +421,15 @@ relpTcpSetAuthMode(relpTcp_t *pThis, relpAuthMode_t authmode)
 	LEAVE_RELPFUNC;
 }
 
+relpRetVal
+relpTcpSetConnTimeout(relpTcp_t *pThis, int connTimeout)
+{
+	ENTER_RELPFUNC;
+	RELPOBJ_assert(pThis, Tcp);
+	pThis->connTimeout = connTimeout;
+	LEAVE_RELPFUNC;
+}
+
 relpRetVal
 relpTcpSetGnuTLSPriString(relpTcp_t *pThis, char *pristr)
 {
@@ -563,7 +573,9 @@ finalize_it:
 	LEAVE_RELPFUNC;
 }
 
-
+#pragma GCC diagnostic push
+/* per https://lists.gnupg.org/pipermail/gnutls-help/2004-August/000154.html This is expected */
+#pragma GCC diagnostic ignored "-Wint-to-pointer-cast"
 static relpRetVal
 relpTcpAcceptConnReqInitTLS(relpTcp_t *pThis, relpSrv_t *pSrv)
 {
@@ -615,6 +627,7 @@ relpTcpAcceptConnReqInitTLS(relpTcp_t *pThis, relpSrv_t *pSrv)
 finalize_it:
   	LEAVE_RELPFUNC;
 }
+#pragma GCC diagnostic pop
 #endif /* #ifdef ENABLE_TLS */
 
 /* Enable KEEPALIVE handling on the socket.  */
@@ -690,6 +703,19 @@ done:
   	return;
 }
 
+/* a portable way to put the current thread asleep. Note that
+ * using the sleep() API family may result in the whole process
+ * to be put asleep on some platforms.
+ */
+static void
+doSleep(int iSeconds, int iuSeconds)
+{
+	struct timeval tvSelectTimeout;
+	tvSelectTimeout.tv_sec = iSeconds;
+	tvSelectTimeout.tv_usec = iuSeconds; /* micro seconds */
+	select(0, NULL, NULL, NULL, &tvSelectTimeout);
+}
+
 /* accept an incoming connection request, sock provides the socket on which we can
  * accept the new session.
  * rgerhards, 2008-03-17
@@ -708,7 +734,12 @@ relpTcpAcceptConnReq(relpTcp_t **ppThis, int sock, relpSrv_t *pSrv)
 	assert(ppThis != NULL);
 
 	iNewSock = accept(sock, (struct sockaddr*) &addr, &addrlen);
+	int errnosave = errno;
 	if(iNewSock < 0) {
+		pSrv->pEngine->dbgprint("error during accept, sleeping 20ms: %s\n",
+			strerror(errnosave));
+		doSleep(0, 20000);
+		pSrv->pEngine->dbgprint("END SLEEP\n");
 		ABORT_FINALIZE(RELP_RET_ACCEPT_ERR);
 	}
 
@@ -778,7 +809,7 @@ GenFingerprintStr(char *pFingerprint, int sizeFingerprint, char *fpBuf)
 
 /* Check the peer's ID in fingerprint auth mode. */
 static int
-relpTcpChkPeerFingerprint(relpTcp_t *pThis, gnutls_x509_crt cert)
+relpTcpChkPeerFingerprint(relpTcp_t *pThis, gnutls_x509_crt_t cert)
 {
 	int r = 0;
 	int i;
@@ -1070,7 +1101,7 @@ relpTcpChkOnePeerName(relpTcp_t *pThis, char *peername, int *pbFoundPositiveMatc
  * Note that non-0 is also returned if no CN is found.
  */
 static int
-relpTcpGetCN(relpTcp_t *pThis, gnutls_x509_crt cert, char *namebuf, int lenNamebuf)
+relpTcpGetCN(relpTcp_t *pThis, gnutls_x509_crt_t cert, char *namebuf, int lenNamebuf)
 {
 	int r;
 	int gnuRet;
@@ -1129,7 +1160,7 @@ done:
 
 /* Check the peer's ID in name auth mode. */
 static int
-relpTcpChkPeerName(relpTcp_t *pThis, gnutls_x509_crt cert)
+relpTcpChkPeerName(relpTcp_t *pThis, gnutls_x509_crt_t cert)
 {
 	int r = 0;
 	int ret;
@@ -1203,9 +1234,9 @@ relpTcpVerifyCertificateCallback(gnutls_session_t session)
 {
 	int r = 0;
 	relpTcp_t *pThis;
-	const gnutls_datum *cert_list;
+	const gnutls_datum_t *cert_list;
 	unsigned int list_size = 0;
-	gnutls_x509_crt cert;
+	gnutls_x509_crt_t cert;
 	int bMustDeinitCert = 0;
 
 	pThis = (relpTcp_t*) gnutls_session_get_ptr(session);
@@ -1265,6 +1296,10 @@ relpTcpLstnInitTLS(relpTcp_t *pThis)
 	ENTER_RELPFUNC;
 	RELPOBJ_assert(pThis, Tcp);
 
+	#if GNUTLS_VERSION_NUMBER <= 0x020b00
+	/* gcry_control must be called first, so that the thread system is correctly set up */
+	gcry_control (GCRYCTL_SET_THREAD_CBS, &gcry_threads_pthread);
+	#endif
 	gnutls_global_init();
 	/* uncomment for (very intense) debug help
 	 * gnutls_global_set_log_function(logFunction);
@@ -1495,6 +1530,8 @@ relpTcpRcv(relpTcp_t *pThis, relpOctet_t *pRcvBuf, ssize_t *pLenBuf)
 	} else {
 #endif /* #ifdef ENABLE_TLS */
 		*pLenBuf = recv(pThis->sock, pRcvBuf, *pLenBuf, MSG_DONTWAIT);
+		pThis->pEngine->dbgprint("relpTcpRcv: read %zd bytes from sock %d\n",
+			*pLenBuf, pThis->sock);
 #ifdef ENABLE_TLS
 	}
 #endif /* #ifdef ENABLE_TLS */
@@ -1566,8 +1603,11 @@ relpTcpSend(relpTcp_t *pThis, relpOctet_t *pBuf, ssize_t *pLenBuf)
 	} else {
 #endif /* #ifdef ENABLE_TLS */
 		written = send(pThis->sock, pBuf, *pLenBuf, 0);
+		const int errno_save = errno;
+		pThis->pEngine->dbgprint("relpTcpSend: sock %d, lenbuf %zd, send returned %d [errno %d]\n",
+			(int)pThis->sock, *pLenBuf, (int) written, errno_save);
 		if(written == -1) {
-			switch(errno) {
+			switch(errno_save) {
 				case EAGAIN:
 				case EINTR:
 					/* this is fine, just retry... */
@@ -1588,6 +1628,9 @@ finalize_it:
 }
 
 #ifdef ENABLE_TLS
+#pragma GCC diagnostic push /* we need to disable a warning below */
+/* per https://lists.gnupg.org/pipermail/gnutls-help/2004-August/000154.html This is expected */
+#pragma GCC diagnostic ignored "-Wint-to-pointer-cast"
 /* this is only called for client-initiated sessions */
 static relpRetVal
 relpTcpConnectTLSInit(relpTcp_t *pThis)
@@ -1597,7 +1640,22 @@ relpTcpConnectTLSInit(relpTcp_t *pThis)
 	ENTER_RELPFUNC;
 	RELPOBJ_assert(pThis, Tcp);
 
+	/* We expect a non blocking socket to establish a tls session */
+	if((sockflags = fcntl(pThis->sock, F_GETFL)) != -1) {
+		sockflags &= ~O_NONBLOCK;
+		sockflags = fcntl(pThis->sock, F_SETFL, sockflags);
+	}
+
+	if(sockflags == -1) {
+		pThis->pEngine->dbgprint("error %d unsetting fcntl(O_NONBLOCK) on relp socket", errno);
+		ABORT_FINALIZE(RELP_RET_IO_ERR);
+	}
+
 	if(!called_gnutls_global_init) {
+		#if GNUTLS_VERSION_NUMBER <= 0x020b00
+		/* gcry_control must be called first, so that the thread system is correctly set up */
+		gcry_control (GCRYCTL_SET_THREAD_CBS, &gcry_threads_pthread);
+		#endif
 		gnutls_global_init();
 		/* uncomment for (very intense) debug help
 		 * gnutls_global_set_log_function(logFunction);
@@ -1686,6 +1744,7 @@ relpTcpConnectTLSInit(relpTcp_t *pThis)
 finalize_it:
 	LEAVE_RELPFUNC;
 }
+#pragma GCC diagnostic pop
 #endif /* #ifdef ENABLE_TLS */
 
 /* open a connection to a remote host (server).
@@ -1698,6 +1757,7 @@ relpTcpConnect(relpTcp_t *pThis, int family, unsigned char *port, unsigned char
 	struct addrinfo *res = NULL;
 	struct addrinfo hints;
 	struct addrinfo *reslocal = NULL;
+	struct pollfd pfd;
 
 	ENTER_RELPFUNC;
 	RELPOBJ_assert(pThis, Tcp);
@@ -1727,10 +1787,27 @@ relpTcpConnect(relpTcp_t *pThis, int family, unsigned char *port, unsigned char
 		}
 	}
 
-	if(connect(pThis->sock, res->ai_addr, res->ai_addrlen) != 0) {
+	fcntl(pThis->sock, F_SETFL, O_NONBLOCK);
+	connect(pThis->sock, res->ai_addr, res->ai_addrlen);
+
+	pfd.fd = pThis->sock;
+	pfd.events = POLLOUT;
+
+	if (poll(&pfd, 1, pThis->connTimeout * 1000) != 1) {
+		pThis->pEngine->dbgprint("connection timed out after %d seconds\n", pThis->connTimeout);
+		ABORT_FINALIZE(RELP_RET_TIMED_OUT);
+	}
+
+	int so_error;
+	socklen_t len = sizeof so_error;
+
+	getsockopt(pThis->sock, SOL_SOCKET, SO_ERROR, &so_error, &len);
+	if (so_error != 0) {
+		pThis->pEngine->dbgprint("socket has an error %d\n", so_error);
 		ABORT_FINALIZE(RELP_RET_IO_ERR);
 	}
 
+
 #ifdef ENABLE_TLS
 	if(pThis->bEnableTLS) {
 		CHKRet(relpTcpConnectTLSInit(pThis));
@@ -1795,29 +1872,24 @@ finalize_it:
  * otherwise.
  */
 int
-relpTcpWaitWriteable(relpTcp_t *pThis, struct timespec *tTimeout)
+relpTcpWaitWriteable(relpTcp_t *const pThis, struct timespec *const tTimeout)
 {
 	int r;
-	fd_set writefds;
 	struct timespec tCurr; /* current time */
-	struct timeval tvSelect;
+	struct pollfd pfd;
 
 	clock_gettime(CLOCK_REALTIME, &tCurr);
-	tvSelect.tv_sec = tTimeout->tv_sec - tCurr.tv_sec;
-	tvSelect.tv_usec = (tTimeout->tv_nsec - tCurr.tv_nsec) / 1000000;
-	if(tvSelect.tv_usec < 0) {
-		tvSelect.tv_usec += 1000000;
-		tvSelect.tv_sec--;
-	}
-	if(tvSelect.tv_sec < 0) {
+	const int timeout =   (tTimeout->tv_sec - tCurr.tv_sec) * 1000
+			    + (tTimeout->tv_nsec - tCurr.tv_nsec) / 1000000000;
+	if(timeout < 0) {
 		r = 0; goto done;
 	}
 
-	FD_ZERO(&writefds);
-	FD_SET(pThis->sock, &writefds);
-	pThis->pEngine->dbgprint("librelp: telpTcpWaitWritable doing select() "
-		"on fd %d, timoeut %lld.%lld\n", pThis->sock,
-		(long long) tTimeout->tv_sec, (long long) tTimeout->tv_nsec);
-	r = select(pThis->sock+1, NULL, &writefds, NULL, &tvSelect);
+	pThis->pEngine->dbgprint("librelp: telpTcpWaitWritable doing poll() "
+		"on fd %d, timoeut %d\n", pThis->sock, timeout);
+
+	pfd.fd = pThis->sock;
+	pfd.events = POLLOUT;
+	r = poll(&pfd, 1, timeout);
 done:	return r;
 }
